@@ -1,35 +1,36 @@
 """Extensions to unittest for web frameworks.
 
-Use the WebCase.getPage method to request a page from your HTTP server.
+Use the :py:meth:`WebCase.getPage` method to request a page
+from your HTTP server.
+
 Framework Integration
 =====================
 If you have control over your server process, you can handle errors
 in the server-side of the HTTP conversation a bit better. You must run
-both the client (your WebCase tests) and the server in the same process
-(but in separate threads, obviously).
+both the client (your :py:class:`WebCase` tests) and the server in the
+same process (but in separate threads, obviously).
 When an error occurs in the framework, call server_error. It will print
 the traceback to stdout, and keep any assertions you have from running
 (the assumption is that, if the server errors, the page output will not
 be of further significance to your tests).
 """
 
-from __future__ import absolute_import, division, print_function
-__metaclass__ = type
-
+import contextlib
+import functools
+import http.client
+import json
+import os
 import pprint
 import re
 import socket
 import sys
 import time
 import traceback
-import os
-import json
-import unittest
+import unittest  # pylint: disable=deprecated-module,preferred-module
+import urllib.parse
 import warnings
 
-from six.moves import range, http_client, map, urllib_parse
-import six
-
+import jaraco.functools
 from more_itertools.more import always_iterable
 
 
@@ -51,6 +52,7 @@ def interface(host):
 try:
     # Jython support
     if sys.platform[:4] == 'java':
+
         def getchar():
             """Get a key press."""
             # Hopefully this is enough
@@ -64,8 +66,8 @@ try:
             return msvcrt.getch()
 except ImportError:
     # Unix getchr
-    import tty
     import termios
+    import tty
 
     def getchar():
         """Get a key press."""
@@ -101,7 +103,7 @@ class WebCase(unittest.TestCase):
 
     HOST = '127.0.0.1'
     PORT = 8000
-    HTTP_CONN = http_client.HTTPConnection
+    HTTP_CONN = http.client.HTTPConnection
     PROTOCOL = 'HTTP/1.1'
 
     scheme = 'http'
@@ -120,10 +122,10 @@ class WebCase(unittest.TestCase):
     def _Conn(self):
         """Return HTTPConnection or HTTPSConnection based on self.scheme.
 
-        * from http.client.
+        * from :py:mod:`python:http.client`.
         """
-        cls_name = '{scheme}Connection'.format(scheme=self.scheme.upper())
-        return getattr(http_client, cls_name)
+        cls_name = f'{self.scheme.upper()}Connection'
+        return getattr(http.client, cls_name)
 
     def get_conn(self, auto_open=False):
         """Return a connection to our HTTP server."""
@@ -142,20 +144,16 @@ class WebCase(unittest.TestCase):
         As this class only allows for a single open connection, if
         self already has an open connection, it will be closed.
         """
-        try:
+        with contextlib.suppress(TypeError, AttributeError):
             self.HTTP_CONN.close()
-        except (TypeError, AttributeError):
-            pass
 
         self.HTTP_CONN = (
-            self.get_conn(auto_open=auto_open)
-            if on
-            else self._Conn
+            self.get_conn(auto_open=auto_open) if on else self._Conn
         )
 
     @property
-    def persistent(self):  # noqa: D401; irrelevant for properties
-        """Presense of the persistent HTTP connection."""
+    def persistent(self):
+        """Presence of the persistent HTTP connection."""
         return hasattr(self.HTTP_CONN, '__class__')
 
     @persistent.setter
@@ -171,10 +169,17 @@ class WebCase(unittest.TestCase):
         return interface(self.HOST)
 
     def getPage(
-        self, url, headers=None, method='GET', body=None,
-        protocol=None, raise_subcls=None,
+        self,
+        url,
+        headers=None,
+        method='GET',
+        body=None,
+        protocol=None,
+        raise_subcls=(),
     ):
-        """Open the url with debugging support. Return status, headers, body.
+        """Open the url with debugging support.
+
+        Return status, headers, body.
 
         url should be the identifier passed to the server, typically a
         server-absolute path and query string (sent between method and
@@ -182,46 +187,52 @@ class WebCase(unittest.TestCase):
         enabled in the server.
 
         If the application under test generates absolute URIs, be sure
-        to wrap them first with strip_netloc::
+        to wrap them first with :py:func:`strip_netloc`::
 
-            class MyAppWebCase(WebCase):
-                def getPage(url, *args, **kwargs):
-                    super(MyAppWebCase, self).getPage(
-                        cheroot.test.webtest.strip_netloc(url),
-                        *args, **kwargs
-                    )
+            >>> class MyAppWebCase(WebCase):
+            ...     def getPage(url, *args, **kwargs):
+            ...         super(MyAppWebCase, self).getPage(
+            ...             cheroot.test.webtest.strip_netloc(url),
+            ...             *args, **kwargs
+            ...         )
 
-        `raise_subcls` must be a tuple with the exceptions classes
-        or a single exception class that are not going to be considered
-        a socket.error regardless that they were are subclass of a
-        socket.error and therefore not considered for a connection retry.
+        ``raise_subcls`` is passed through to :py:func:`openURL`.
         """
         ServerError.on = False
 
-        if isinstance(url, six.text_type):
+        if isinstance(url, str):
             url = url.encode('utf-8')
-        if isinstance(body, six.text_type):
+        if isinstance(body, str):
             body = body.encode('utf-8')
 
+        # for compatibility, support raise_subcls is None
+        raise_subcls = raise_subcls or ()
+
         self.url = url
         self.time = None
         start = time.time()
         result = openURL(
-            url, headers, method, body, self.HOST, self.PORT,
-            self.HTTP_CONN, protocol or self.PROTOCOL,
-            raise_subcls, ssl_context=self.ssl_context,
+            url,
+            headers,
+            method,
+            body,
+            self.HOST,
+            self.PORT,
+            self.HTTP_CONN,
+            protocol or self.PROTOCOL,
+            raise_subcls=raise_subcls,
+            ssl_context=self.ssl_context,
         )
         self.time = time.time() - start
         self.status, self.headers, self.body = result
 
         # Build a list of request cookies from the previous response cookies.
         self.cookies = [
-            ('Cookie', v) for k, v in self.headers
-            if k.lower() == 'set-cookie'
+            ('Cookie', v) for k, v in self.headers if k.lower() == 'set-cookie'
         ]
 
         if ServerError.on:
-            raise ServerError()
+            raise ServerError
         return result
 
     @NonDataProperty
@@ -239,13 +250,14 @@ class WebCase(unittest.TestCase):
                 'Interactive test failure interceptor support via '
                 'WEBTEST_INTERACTIVE environment variable is deprecated.',
                 DeprecationWarning,
+                stacklevel=1,
             )
         return is_interactive
 
     console_height = 30
 
-    def _handlewebError(self, msg):
-        print('')
+    def _handlewebError(self, msg):  # noqa: C901  # FIXME
+        print()
         print('    ERROR: %s' % msg)
 
         if not self.interactive:
@@ -260,7 +272,7 @@ class WebCase(unittest.TestCase):
         sys.stdout.flush()
         while True:
             i = getchar().upper()
-            if not isinstance(i, type('')):
+            if not isinstance(i, str):
                 i = i.decode('ascii')
             if i not in 'BHSUIRX':
                 continue
@@ -299,11 +311,7 @@ class WebCase(unittest.TestCase):
 
     def status_matches(self, expected):
         """Check whether actual status matches expected."""
-        actual = (
-            self.status_code
-            if isinstance(expected, int) else
-            self.status
-        )
+        actual = self.status_code if isinstance(expected, int) else self.status
         return expected == actual
 
     def assertStatus(self, status, msg=None):
@@ -315,9 +323,8 @@ class WebCase(unittest.TestCase):
         if any(map(self.status_matches, always_iterable(status))):
             return
 
-        tmpl = 'Status {self.status} does not match {status}'
-        msg = msg or tmpl.format(**locals())
-        self._handlewebError(msg)
+        tmpl = f'Status {self.status} does not match {status}'
+        self._handlewebError(msg or tmpl)
 
     def assertHeader(self, key, value=None, msg=None):
         """Fail if (key, [value]) not in self.headers."""
@@ -333,6 +340,7 @@ class WebCase(unittest.TestCase):
             else:
                 msg = '%r:%r not in headers' % (key, value)
         self._handlewebError(msg)
+        return None
 
     def assertHeaderIn(self, key, values, msg=None):
         """Fail if header indicated by key doesn't have one of the values."""
@@ -346,6 +354,7 @@ class WebCase(unittest.TestCase):
         if msg is None:
             msg = '%(key)r not in %(values)r' % vars()
         self._handlewebError(msg)
+        return None
 
     def assertHeaderItemValue(self, key, value, msg=None):
         """Fail if the header does not contain the specified value."""
@@ -357,6 +366,7 @@ class WebCase(unittest.TestCase):
         if msg is None:
             msg = '%r not in %r' % (value, header_values)
         self._handlewebError(msg)
+        return None
 
     def assertNoHeader(self, key, msg=None):
         """Fail if key in self.headers."""
@@ -379,18 +389,19 @@ class WebCase(unittest.TestCase):
 
     def assertBody(self, value, msg=None):
         """Fail if value != self.body."""
-        if isinstance(value, six.text_type):
+        if isinstance(value, str):
             value = value.encode(self.encoding)
         if value != self.body:
             if msg is None:
                 msg = 'expected body:\n%r\n\nactual body:\n%r' % (
-                    value, self.body,
+                    value,
+                    self.body,
                 )
             self._handlewebError(msg)
 
     def assertInBody(self, value, msg=None):
         """Fail if value not in self.body."""
-        if isinstance(value, six.text_type):
+        if isinstance(value, str):
             value = value.encode(self.encoding)
         if value not in self.body:
             if msg is None:
@@ -399,7 +410,7 @@ class WebCase(unittest.TestCase):
 
     def assertNotInBody(self, value, msg=None):
         """Fail if value in self.body."""
-        if isinstance(value, six.text_type):
+        if isinstance(value, str):
             value = value.encode(self.encoding)
         if value in self.body:
             if msg is None:
@@ -408,7 +419,7 @@ class WebCase(unittest.TestCase):
 
     def assertMatchesBody(self, pattern, msg=None, flags=0):
         """Fail if value (a regex pattern) is not in self.body."""
-        if isinstance(pattern, six.text_type):
+        if isinstance(pattern, str):
             pattern = pattern.encode(self.encoding)
         if re.search(pattern, self.body, flags) is None:
             if msg is None:
@@ -427,7 +438,7 @@ def cleanHeaders(headers, method, body, host, port):
     # Add the required Host request header if not present.
     # [This specifies the host:port of the server, not the client.]
     found = False
-    for k, v in headers:
+    for k, _v in headers:
         if k.lower() == 'host':
             found = True
             break
@@ -440,7 +451,7 @@ def cleanHeaders(headers, method, body, host, port):
     if method in methods_with_bodies:
         # Stick in default type and length headers if not present
         found = False
-        for k, v in headers:
+        for k, _v in headers:
             if k.lower() == 'content-type':
                 found = True
                 break
@@ -457,89 +468,77 @@ def shb(response):
     """Return status, headers, body the way we like from a response."""
     resp_status_line = '%s %s' % (response.status, response.reason)
 
-    if not six.PY2:
-        return resp_status_line, response.getheaders(), response.read()
-
-    h = []
-    key, value = None, None
-    for line in response.msg.headers:
-        if line:
-            if line[0] in ' \t':
-                value += line.strip()
-            else:
-                if key and value:
-                    h.append((key, value))
-                key, value = line.split(':', 1)
-                key = key.strip()
-                value = value.strip()
-    if key and value:
-        h.append((key, value))
-
-    return resp_status_line, h, response.read()
+    return resp_status_line, response.getheaders(), response.read()
 
 
-def openURL(
-    url, headers=None, method='GET', body=None,
-    host='127.0.0.1', port=8000, http_conn=http_client.HTTPConnection,
-    protocol='HTTP/1.1', raise_subcls=None, ssl_context=None,
+def openURL(*args, raise_subcls=(), **kwargs):
+    """
+    Open a URL, retrying when it fails.
+
+    Specify ``raise_subcls`` (class or tuple of classes) to exclude
+    those socket.error subclasses from being suppressed and retried.
+    """
+    opener = functools.partial(_open_url_once, *args, **kwargs)
+
+    def on_exception():
+        exc = sys.exc_info()[1]
+        if isinstance(exc, raise_subcls):
+            raise exc
+        time.sleep(0.5)
+
+    # Try up to 10 times
+    return jaraco.functools.retry_call(
+        opener,
+        retries=9,
+        cleanup=on_exception,
+        trap=socket.error,
+    )
+
+
+def _open_url_once(
+    url,
+    headers=None,
+    method='GET',
+    body=None,
+    host='127.0.0.1',
+    port=8000,
+    http_conn=http.client.HTTPConnection,
+    protocol='HTTP/1.1',
+    ssl_context=None,
 ):
-    """
-    Open the given HTTP resource and return status, headers, and body.
-
-    `raise_subcls` must be a tuple with the exceptions classes
-    or a single exception class that are not going to be considered
-    a socket.error regardless that they were are subclass of a
-    socket.error and therefore not considered for a connection retry.
-    """
+    """Open the given HTTP resource and return status, headers, and body."""
     headers = cleanHeaders(headers, method, body, host, port)
 
-    # Trying 10 times is simply in case of socket errors.
-    # Normal case--it should run once.
-    for trial in range(10):
-        try:
-            # Allow http_conn to be a class or an instance
-            if hasattr(http_conn, 'host'):
-                conn = http_conn
-            else:
-                kw = {}
-                if ssl_context:
-                    kw['context'] = ssl_context
-                conn = http_conn(interface(host), port, **kw)
-
-            conn._http_vsn_str = protocol
-            conn._http_vsn = int(''.join([x for x in protocol if x.isdigit()]))
-
-            if not six.PY2 and isinstance(url, bytes):
-                url = url.decode()
-            conn.putrequest(
-                method.upper(), url, skip_host=True,
-                skip_accept_encoding=True,
-            )
-
-            for key, value in headers:
-                conn.putheader(key, value.encode('Latin-1'))
-            conn.endheaders()
-
-            if body is not None:
-                conn.send(body)
-
-            # Handle response
-            response = conn.getresponse()
-
-            s, h, b = shb(response)
-
-            if not hasattr(http_conn, 'host'):
-                # We made our own conn instance. Close it.
-                conn.close()
-
-            return s, h, b
-        except socket.error as e:
-            if raise_subcls is not None and isinstance(e, raise_subcls):
-                raise
-            else:
-                time.sleep(0.5)
-                if trial == 9:
-                    raise
+    # Allow http_conn to be a class or an instance
+    if hasattr(http_conn, 'host'):
+        conn = http_conn
+    else:
+        kw = {}
+        if ssl_context:
+            kw['context'] = ssl_context
+        conn = http_conn(interface(host), port, **kw)
+    conn._http_vsn_str = protocol
+    conn._http_vsn = int(''.join([x for x in protocol if x.isdigit()]))
+    if isinstance(url, bytes):
+        url = url.decode()
+    conn.putrequest(
+        method.upper(),
+        url,
+        skip_host=True,
+        skip_accept_encoding=True,
+    )
+    for key, value in headers:
+        conn.putheader(key, value.encode('Latin-1'))
+    conn.endheaders()
+    if body is not None:
+        conn.send(body)
+    # Handle response
+    response = conn.getresponse()
+    s, h, b = shb(response)
+    if not hasattr(http_conn, 'host'):
+        # We made our own conn instance. Close it.
+        conn.close()
+    return s, h, b
 
 
 def strip_netloc(url):
@@ -549,7 +548,11 @@ def strip_netloc(url):
     server-absolute portion.
 
     Useful for wrapping an absolute-URI for which only the
-    path is expected (such as in calls to getPage).
+    path is expected (such as in calls to :py:meth:`WebCase.getPage`).
+
+    .. testsetup::
+
+       from cheroot.test.webtest import strip_netloc
 
     >>> strip_netloc('https://google.com/foo/bar?bing#baz')
     '/foo/bar?bing'
@@ -560,10 +563,10 @@ def strip_netloc(url):
     >>> strip_netloc('/foo/bar?bing#baz')
     '/foo/bar?bing'
     """
-    parsed = urllib_parse.urlparse(url)
-    scheme, netloc, path, params, query, fragment = parsed
+    parsed = urllib.parse.urlparse(url)
+    _scheme, _netloc, path, params, query, _fragment = parsed
     stripped = '', '', path, params, query, ''
-    return urllib_parse.urlunparse(stripped)
+    return urllib.parse.urlunparse(stripped)
 
 
 # Add any exceptions which your web framework handles
@@ -594,8 +597,7 @@ def server_error(exc=None):
 
     if ignore_all or exc[0] in ignored_exceptions:
         return False
-    else:
-        ServerError.on = True
-        print('')
-        print(''.join(traceback.format_exception(*exc)))
-        return True
+    ServerError.on = True
+    print()
+    print(''.join(traceback.format_exception(*exc)))
+    return True
